#include <vnet_admin.hh>
#include <vnet_table.hh>
#include <vnet_filter.hh>

#include <boost/bind.hpp>

using namespace vnet::yarp;

std::string get_string(const ::yarp::os::Bottle &b, int n)
{
	return b.get(n).asString().c_str();
}

std::string get_string_(const ::yarp::os::Bottle &b, int n)
{
	std::ostringstream oss;
	yarp::os::Value value = b.get(n);
	if (value.isString())
		return value.asString().c_str();
		
	if (value.isDouble()) {
		oss << value.asDouble();
		return oss.str();
	}

	if (value.isInt()) {
		oss << value.asInt();
		return oss.str();
	}

	throw std::runtime_error("string expected");
}

admin::admin(table & m_table, filter_factory& factory):
	m_table(m_table), factory(factory), quit_(false)
{
	in.open("/vnet/admin/in");
	out.open("/vnet/admin/out");

	dispatch["add"] = boost::bind(&admin::add, this, _1);
	dispatch["rm"] = boost::bind(&admin::rm, this, _1);
	dispatch["remove"] = boost::bind(&admin::rm, this, _1);
	dispatch["print"] = boost::bind(&admin::print, this, _1);
	dispatch["list"] = boost::bind(&admin::print, this, _1);
	dispatch["quit"] = boost::bind(&admin::quit, this, _1);
	dispatch["clear"] = boost::bind(&admin::clear, this, _1);
}

bool 
admin::add(const ::yarp::os::Bottle& b)
{
	int size = b.size();
	int idx = 1;
	std::string first = get_string(b, idx);
	bool is_bi;
	idx ++;

	std::string src, dst;

	if (first == "bi") {
		is_bi = true;
		src = get_string(b, idx);
		idx ++;
		dst = get_string(b, idx);
		idx ++;
	} else {
		is_bi = false;
		src = first;
		dst = get_string(b, idx);
		idx ++;
	}

	std::string filter = get_string(b, idx);
	idx ++;

	std::vector<std::string> argv;
	for (; idx < size; ++idx)
		argv.push_back(get_string_(b, idx));

	filter_base* f = factory.make_filter(filter, argv);
	if (! f)
		return false;

	boost::lock_guard<table> lk(m_table);
	m_table.add(src, dst, f, is_bi);
	return true;
}

bool 
admin::rm(const ::yarp::os::Bottle& b)
{
	int idx = 1;
	std::string first = get_string(b, idx);
	bool is_bi;
	idx ++;

	std::string src, dst;

	if (first == "bi") {
		is_bi = true;
		src = get_string(b, idx);
		idx ++;
		dst = get_string(b, idx);
		idx ++;
	} else {
		is_bi = false;
		src = first;
		dst = get_string(b, idx);
		idx ++;
	}

	int index = b.get(idx).asInt();
	idx ++;

	boost::lock_guard<table> lk(m_table);
	m_table.remove(src, dst, index, is_bi);
	return true;
}

bool
admin::print(const ::yarp::os::Bottle& b)
{
	bool all = (b.size() == 1);
	std::string src, dst;

	if (!all) {
		src = get_string(b, 1);
		dst = get_string(b, 2);
	}


	std::ostringstream oss;
	{
	boost::lock_guard<table> lk(m_table);
	if (all)
		oss << m_table.repr();
	else {
		const table::filter_array& arr = m_table.get(src, dst);
		for (size_t i = 0; i < arr.size(); i++)
			oss << i << ": " << arr[i]->repr() << '\n';
		}
	}

	::yarp::os::Bottle& res = out.prepare();
	res.clear();
	res.addString("OK");
	res.addString(oss.str().c_str());
	out.write();
	return true;
}

bool 
admin::clear(const ::yarp::os::Bottle& b)
{
	int idx = 1;
	std::string first = get_string(b, idx);
	bool is_bi;
	idx ++;

	std::string src, dst;

	if (first == "bi") {
		is_bi = true;
		src = get_string(b, idx);
		idx ++;
		dst = get_string(b, idx);
		idx ++;
	} else {
		is_bi = false;
		src = first;
		dst = get_string(b, idx);
		idx ++;
	}

	boost::lock_guard<table> lk(m_table);
	m_table.clear(src, dst, is_bi);
	return true;
}

bool
admin::quit(const ::yarp::os::Bottle& b)
{
	(void) b;
	quit();
	return true;
}

bool
admin::process()
{
	::yarp::os::Bottle* res = in.read(true);
	if (res == NULL)
		return false;

	if (res->size() == 0)
		return false;

	std::string keyword = get_string(*res, 0);
	dispatch_table::const_iterator it = dispatch.find(keyword);
	if (it == dispatch.end())
		return true;
	else
		return it->second(*res);
}

void
admin::process_all()
{
	bool ok = true;
	while ( ok && !quit_ ) 
		ok = process();
}
