#include <iostream>
#include <vnet_proxy.hh>
#include <vnet_morse.hh>
#include <vnet_filter.hh>
#include <vnet_table.hh>
#include <vnet_admin.hh>

#include <boost/thread.hpp>
#include <boost/bind.hpp>

using namespace vnet::yarp;

namespace po = boost::program_options;

int main(int argc, char ** argv)
{
	po::options_description desc("Allowed options");
	desc.add_options()
	("filter", po::value<std::string>()->default_value("pass"), "use the selected filter")
	("help,h", "produce help message")
	("host", po::value<std::string>()->default_value("localhost"), "host (morse)")
	("port", po::value<std::string>()->default_value("4000"), "port (morse)")
	;

	po::variables_map vm;
	po::parsed_options parsed = po::command_line_parser(argc, argv).
			options(desc).allow_unregistered().run();
	po::store(parsed, vm);
	po::notify(vm);

	if (vm.count("help") or !vm.count("filter")) {
		std::cout << desc << std::endl;
		return 0;
	}

	vnet::morse_world::init_instance(vm["host"].as<std::string>(),
									 vm["port"].as<std::string>());

	std::vector<std::string> m_argv = 
		po::collect_unrecognized(parsed.options, po::include_positional);
	yarp::os::Network::init();

	filter_factory factory;
	filter_base* filter = factory.make_filter(vm["filter"].as<std::string>(), m_argv);

	table mtable(filter);

	admin adm(mtable, factory);
	boost::thread thr(boost::bind(&admin::process_all, boost::ref(adm)));

	vnetProxy proxy(mtable);

	while (1) {
		proxy.run();
	}

	adm.quit();
	thr.join();

	yarp::os::Network::fini();
}
